"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Compiler = void 0;
const Readers_1 = require("./Readers");
class Compiler {
    /**
     * Convert all first-phase-compilation tokens to function and string tokens.
     * @param code - Code to be compiled.
     * @param dots - Whether enable "rs" syntax.
     * @param trim - Whether trim all code lines.
     * @returns {TemporalToken[]}
     */
    pack(code, dots = true, trim = true) {
        const groups = (dots === true ? Readers_1.ReaderV1 : Readers_1.ReaderV2).read(code, trim);
        const tokens = [];
        let temp_token = {
            value: "",
            type: "TEXT"
        };
        for (let i = 0; i < groups.length; i++) {
            const token = groups[i];
            if (`${token.type}`.includes("FUNCTION")) {
                if (groups[i + 1] === undefined ||
                    groups[i + 1].type === "TEXT") {
                    temp_token.value += token.value;
                    temp_token.type = "FN";
                    tokens.push(temp_token);
                    temp_token = {
                        value: "",
                        type: "TEXT"
                    };
                }
                else
                    temp_token.value += token.value;
            }
            else {
                if (groups[i + 1] === undefined ||
                    `${groups[i + 1].type}`.includes("FUNCTION")) {
                    temp_token.value += token.value;
                    tokens.push(temp_token);
                    temp_token = {
                        value: "",
                        type: "TEXT"
                    };
                }
                else
                    temp_token.value += token.value;
            }
        }
        return tokens;
    }
    /**
     * Converts a string token content to a function token.
     * @param input - Input to be converted.
     * @returns {FunctionToken}
     */
    restructToken(input) {
        const name = input.match(/\$\w+/g)?.[0] ?? "$__VOID__";
        const inside = input.includes("[") && input.includes("]")
            ? input.slice(input.indexOf("[") + 1, input.lastIndexOf("]"))
            : "$__NULL__";
        const fields = [];
        if (inside && inside !== "$__NULL__") {
            let depth = 0, writing = "";
            const chars = Array.from(inside);
            for (let i = 0; i < chars.length; i++) {
                const char = chars[i];
                if (char === "[")
                    depth++;
                else if (char === "]")
                    depth--;
                if (char === ";" && depth === 0) {
                    if (chars[i + 1] === "]")
                        fields.push("");
                    else
                        fields.push(writing);
                    writing = "";
                    continue;
                }
                writing += char;
            }
            if (writing !== "")
                fields.push(writing);
        }
        return {
            name,
            all: input,
            inside,
            fields
        };
    }
    /**
     * Compiles source code into manipulable tokens.
     * @param code - Code to be compiled.
     * @param dots - Whether use "rs" syntax.
     * @param trim - Whether trim all code lines.
     * @returns {Array<TemporalToken | FunctionToken>}
     */
    remap(code, dots = true, trim = true) {
        const tokens = this.pack(code, dots).map((token) => {
            if (token.type === "TEXT")
                return token;
            else
                return this.restructToken(`${token.value}`);
        });
        return tokens;
    }
}
exports.Compiler = Compiler;
