"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Interpreter = void 0;
const tslib_1 = require("tslib");
const Compiler_1 = require("./Compiler");
const main_1 = require("../main");
const colors_1 = tslib_1.__importDefault(require("colors"));
const ms_1 = tslib_1.__importDefault(require("ms"));
/**
 * Check if the given token is a valid string token.
 * @param token - Token to be validated.
 * @returns {boolean}
 */
function isStringToken(token) {
    return token.type && token.type === "TEXT";
}
/**
 * Removes all JS comments from a text.
 * @param text - Text to work on.
 * @returns {string}
 */
function removeComments(text) {
    return text.split("\n").map((line) => line.split("//")[0]).join("\n");
}
class Interpreter {
    /**
     * Parses easy-api.ts code.
     * @param code Code to parse.
     * @param options Parse extra options.
     */
    async run(code, options) {
        options.comments = options.comments ?? false;
        options.reverse = options.reverse ?? false;
        // Remove JavaScript comments if allowed.
        if (options.comments === true)
            code = removeComments(code);
        // Interpretation start time.
        options.data.setStart(performance.now());
        // Compiling tokens
        let tokens = new Compiler_1.Compiler().remap(code, options.data.server.rs, true);
        const { reverse } = options;
        const codes = [];
        // Reversing code if allowed.
        tokens = reverse ? tokens.reverse() : tokens;
        // Token parsing.
        for (const token of tokens) {
            if (options.data.break) {
                break;
            }
            // Check if the current token is a string.
            if (isStringToken(token)) {
                codes.push(token.value);
            }
            else {
                if (!options.data.functions.has(token.name.toLowerCase())) {
                    throw new main_1.EATS_Error(`"${token.name}" is not a function.`);
                }
                const instruction = options.data.functions.get(token.name.toLowerCase());
                const parsedFields = [];
                const paramsLength = instruction.parameters?.length ?? 0;
                const requiredParams = paramsLength > 0 ? instruction.parameters.filter(param => param.required === true) : [];
                if (paramsLength > 0 && requiredParams.length > token.fields.length) {
                    throw new main_1.EATS_Error([
                        `Function "${token.name}" is not receiving enough parameters!`,
                        `> "${token.name}" expects ${requiredParams.length} parameter${requiredParams.length > 1 ? 's' : ''} at least.`
                    ].join("\n"));
                }
                if (options.debug === true) {
                    console.log(`${colors_1.default.green(new Date().toLocaleString())} | ${colors_1.default.yellow("Parsing")}: ${colors_1.default.bold(instruction.name)}`);
                }
                if (paramsLength > 0 && instruction.compile === true) {
                    for (let i = 0; i < token.fields.length; i++) {
                        const field = token.fields[i];
                        const instructionField = instruction.parameters[i];
                        // Rest parameter validation.
                        if ((instructionField?.rest ?? false) && i !== instruction.parameters.length - 1) {
                            throw new main_1.EATS_Error(`A rest parameter must be the last parameter for a function!`);
                        }
                        const result = await this.run(field, options);
                        options.data.break = result.break;
                        parsedFields.push(typeof (result.code) !== "string" ? "" : result.code //this.#parse(instructionField.type, Unescape(result.code), token)
                        );
                        // Parsing rest parameters.
                        if (i === paramsLength && (instructionField?.rest ?? false)) {
                            const leftFields = token.fields.slice(i + 1);
                            for (const restField of leftFields) {
                                const result = await this.run(restField, options);
                                options.data.break = result.break;
                                parsedFields.push(typeof (result.code) !== "string" ? "" : result.code //this.#parse(instructionField.type, Unescape(result.code), token)
                                );
                            }
                        }
                        /*if (field === " " || field.toUpperCase() === "%SPACE%") {
                            parsedFields.push(" ");
                        } else {
                            
                        }*/
                    }
                }
                else if (paramsLength > 0 && instruction.compile === false) {
                    parsedFields.push(...token.fields.map(x => (0, main_1.Unescape)(x)));
                }
                // Current interpreted function.
                options.data.function = {
                    name: token.name,
                    inside: parsedFields.join(";"),
                    fields: parsedFields
                };
                // Running the function.
                const result = await instruction.run(options.data, parsedFields);
                // Push the function result.
                codes.push(result === undefined ? "" : typeof result === "string" ? (0, main_1.Unescape)(result) : result);
                // Handle the output for this function.
                // this.#handleOutput(codes[-1], instruction);
            }
        }
        options.data.code = reverse ? codes.reverse().join("") : codes.join("");
        return options.data;
    }
    /**
     * Handles the a function output.
     * @param output - The function output to be parsed.
     * @param definition - Function definition.
     * @returns {true}
     */
    #handleOutput(output, definition) {
        const parsedOutput = main_1.Util.parse(output);
        if (definition.returns === main_1.ParamType.String && typeof parsedOutput !== 'string') {
            throw new main_1.EATS_Error(`Function "${definition.name}" expects an string as output but got: ${typeof parsedOutput}`);
        }
        else if (definition.returns === main_1.ParamType.Number && typeof parsedOutput !== 'number') {
            throw new main_1.EATS_Error(`Function "${definition.name}" expects a number as output but got: ${typeof parsedOutput}`);
        }
        else if (definition.returns === main_1.ParamType.Boolean && typeof parsedOutput !== 'boolean') {
            throw new main_1.EATS_Error(`Function "${definition.name}" expects a boolean as output but got: ${typeof parsedOutput}`);
        }
        else if (definition.returns === main_1.ParamType.Time && typeof parsedOutput !== "number") {
            throw new main_1.EATS_Error(`Function "${definition.name}" expects a time as output but got: ${typeof parsedOutput}`);
        }
        else if (definition.returns === main_1.ParamType.BigInt && typeof parsedOutput !== "bigint") {
            throw new main_1.EATS_Error(`Function "${definition.name}" expects a bigint as output but got: ${typeof parsedOutput}`);
        }
        else if (definition.returns === main_1.ParamType.JSON && typeof parsedOutput !== "object") {
            throw new main_1.EATS_Error(`Function "${definition.name}" expects a JSON as output but got: ${typeof parsedOutput}`);
        }
        return true;
    }
    /**
     * Parses an string text to its native type.
     * @param type - Parameter type to parse to.
     * @param thing - Thing to be parsed.
     * @param token - Function source.
     * @param args - Extra arguments.
     * @returns {any}
     */
    #parse(type, thing, token, ...args) {
        if (type === main_1.ParamType.BigInt) {
            return this.#parseBigInt(thing, token);
        }
        else if (type === main_1.ParamType.Boolean) {
            return this.#parseBoolean(thing, token);
        }
        else if (type === main_1.ParamType.Date) {
            return this.#parseDate(token, thing);
        }
        else if (type === main_1.ParamType.JSON) {
            return this.#parseJSON(thing, token);
        }
        else if (type === main_1.ParamType.List) {
            return this.#parseList(thing, args[0], token);
        }
        else if (type === main_1.ParamType.Number) {
            return this.#parseNumber(thing, token);
        }
        else if (type === main_1.ParamType.Time) {
            return this.#parseTime(thing, token);
        }
        else
            return thing;
    }
    /**
     * Parses the given string as boolean.
     * @param bool - The string to be parsed.
     * @param token - Function source.
     * @returns {boolean}
     */
    #parseBoolean(bool, token) {
        const TRUES = ["true", "yes", "1", "!0", "!false"];
        const FALSES = ["false", "no", "0", "!1", "!true"];
        if (!TRUES.includes(bool) && !FALSES.includes(bool)) {
            throw new main_1.EATS_Error([
                `Invalid boolean "${bool}" provided at: "${token.name}"`,
                `> Parameter must be a valid boolean type.`
            ].join("\n"));
        }
        return TRUES.includes(bool);
    }
    /**
     * Parses the given string as big integer.
     * @param bigLike - The string to be parsed.
     * @param token - Function source.
     * @returns {bigint}
     */
    #parseBigInt(bigLike, token) {
        if (!bigLike.endsWith("n")) {
            throw new main_1.EATS_Error([
                `Invalid bigint "${bigLike}" provided at: "${token.name}"`,
                `> Parameter must be a valid big integer type.`
            ].join("\n"));
        }
        const sliced = bigLike.slice(0, -1);
        if (/^[0-9]*$/.test(sliced) === false) {
            throw new main_1.EATS_Error([
                `Invalid bigint "${bigLike}" provided at: "${token.name}"`,
                `> Parameter must be a valid big integer type.`
            ].join("\n"));
        }
        return BigInt(sliced);
    }
    /**
     * Parses the given string as JavaScript date.
     * @param token - Function source.
     * @param date - The string to be parsed.
     * @returns {Date}
     */
    #parseDate(token, date) {
        if (!date)
            return new Date();
        else {
            const dateTime = new Date(date);
            if (dateTime.toString() === "Invalid Date") {
                throw new main_1.EATS_Error([
                    `Invalid date "${date}" provided at: "${token.name}"`,
                    `> Parameter must be a valid date type.`
                ].join("\n"));
            }
            return dateTime;
        }
    }
    /**
     * Parses the given string as JSON object.
     * @param entries - The string to be parsed.
     * @param token - Function source.
     * @returns {Record<unknown, unknown>}
     */
    #parseJSON(entries, token) {
        if (!entries.startsWith("{") || !entries.endsWith("}")) {
            throw new main_1.EATS_Error([
                `Invalid JSON "${entries.slice(0, 6)}..." provided at: "${token.name}"`,
                `> Parameter must be a valid JSON structure.`
            ].join("\n"));
        }
        try {
            return JSON.parse(entries);
        }
        catch {
            throw new main_1.EATS_Error([
                `Invalid JSON "${entries.slice(0, 6)}..." provided at: "${token.name}"`,
                `> Parameter must be a valid JSON structure.`
            ].join("\n"));
        }
    }
    /**
     * Parses a list into a JavaScript array.
     * @param list - The string to be parsed.
     * @param separator - List element separator.
     * @param token - Function source.
     * @returns {string[]}
     */
    #parseList(list, separator, token) {
        return list.split(separator);
    }
    /**
     * Parses the given string as number.
     * @param num - The string to be parsed.
     * @param token - Function source.
     * @returns {number}
     */
    #parseNumber(num, token) {
        const parsedNumber = Number(num);
        if (isNaN(parsedNumber)) {
            throw new main_1.EATS_Error([
                `Invalid number "${num}" provided at: "${token.name}"`,
                `> Parameter must be a valid number type.`
            ].join("\n"));
        }
        return parsedNumber;
    }
    /**
     * Parses the given string as time number.
     * @param time - Time string to be parsed.
     * @param token - Function source.
     * @returns {number}
     */
    #parseTime(time, token) {
        const timeNumber = (0, ms_1.default)(time);
        if (isNaN(timeNumber)) {
            throw new main_1.EATS_Error([
                `Invalid time "${time}" provided at: "${token.name}"`,
                `> Parameter must be a valid number or parseable time string.`
            ].join("\n"));
        }
        return timeNumber;
    }
}
exports.Interpreter = Interpreter;
