"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Util = void 0;
const FunctionManager_1 = require("../managers/FunctionManager");
class Util {
    /**
     * Converts the given arg to boolean.
     * @param value - Value to be parsed.
     * @returns {boolean}
     */
    static booleanify(value) {
        value = value.toLowerCase();
        let r;
        if (["true", "yes", "1", "!0", "!false"].includes(value))
            r = true;
        if (["false", "no", "0", "!1", "!true"].includes(value))
            r = false;
        return r ?? true;
    }
    /**
     * Clones the given argument.
     * @param pairs - Value to be cloned.
     */
    static clone(pairs) {
        if (Array.isArray(pairs)) {
            const list = [];
            pairs.forEach((value) => {
                list.push(Util.clone(value));
            });
            return list;
        }
        else if (pairs instanceof Date) {
            return new Date(pairs);
        }
        else if (pairs instanceof FunctionManager_1.FunctionManager) {
            const manager = new FunctionManager_1.FunctionManager;
            for (const [key, value] of pairs.entries()) {
                manager.set(Util.clone(key), Util.clone(value));
            }
            return manager;
        }
        else if (pairs instanceof Map) {
            const map = new Map;
            for (const [key, value] of pairs.entries()) {
                map.set(Util.clone(key), Util.clone(value));
            }
            return map;
        }
        else if (pairs instanceof RegExp) {
            return new RegExp(pairs);
        }
        else if (pairs instanceof Set) {
            const values = pairs.values();
            return new Set(values);
        }
        else if (pairs instanceof Object) {
            const data = {};
            for (const [key, value] of Object.entries(pairs)) {
                // @ts-ignore
                data[Util.clone(key)] = Util.clone(value);
            }
            return data;
        }
        else
            return pairs;
    }
    /**
     * Fills a canvas text.
     * @param ctx - Canvas context.
     * @param mytext - Text to be filled in the canvas.
     * @param x - X text position.
     * @param y - Y text position.
     * @param width - Textbox width.
     * @param height - Textbox height.
     * @param align - Text align.
     * @param vAlign - Text vertical align.
     */
    static fillText(ctx, mytext, x, y, width, height, align, vAlign) {
        const size = Number(ctx.font.match(/[\d]{1,5}px/g)[0].replace('px', ''));
        if (width <= 0 || height <= 0 || size <= 0) {
            //width or height or font size cannot be 0
            return;
        }
        ;
        // End points
        const xEnd = x + width;
        const yEnd = y + height;
        let txtY = y + height / 2 + size / 2;
        let textanchor;
        if (align === "right") {
            textanchor = xEnd;
            ctx.textAlign = "right";
        }
        else if (align === "left") {
            textanchor = x;
            ctx.textAlign = "left";
        }
        else {
            textanchor = x + width / 2;
            ctx.textAlign = "center";
        }
        //added one-line only auto linebreak feature
        let textarray = [];
        let temptextarray = mytext.split('\n');
        temptextarray.forEach(txtt => {
            let textwidth = ctx.measureText(txtt).width;
            if (textwidth <= width) {
                textarray.push(txtt);
            }
            else {
                let temptext = txtt;
                let linelen = width;
                let textlen;
                let textpixlen;
                let texttoprint;
                textwidth = ctx.measureText(temptext).width;
                while (textwidth > linelen) {
                    textlen = 0;
                    textpixlen = 0;
                    texttoprint = '';
                    while (textpixlen < linelen) {
                        textlen++;
                        texttoprint = temptext.substr(0, textlen);
                        textpixlen = ctx.measureText(temptext.substr(0, textlen)).width;
                    }
                    // Remove last character that was out of the box
                    textlen--;
                    texttoprint = texttoprint.substr(0, textlen);
                    //if statement ensures a new line only happens at a space, and not amidst a word
                    const backup = textlen;
                    if (temptext.substr(textlen, 1) != ' ') {
                        while (temptext.substr(textlen, 1) != ' ' && textlen != 0) {
                            textlen--;
                        }
                        if (textlen == 0) {
                            textlen = backup;
                        }
                        texttoprint = temptext.substr(0, textlen);
                    }
                    texttoprint = texttoprint;
                    temptext = temptext.substr(textlen);
                    textwidth = ctx.measureText(temptext).width;
                    textarray.push(texttoprint);
                }
                if (textwidth > 0) {
                    textarray.push(temptext);
                }
            }
            // end foreach temptextarray
        });
        const charHeight = Util.getTextHeight(ctx, mytext, ctx.font); //close approximation of height with width
        const vheight = charHeight * (textarray.length - 1);
        const negoffset = vheight / 2;
        let debugY = y;
        // Vertical Align
        if (vAlign === 'top') {
            txtY = y + size;
        }
        else if (vAlign === 'bottom') {
            txtY = yEnd - vheight;
            debugY = yEnd;
        }
        else {
            //defaults to center
            debugY = y + height / 2;
            txtY -= negoffset;
        }
        //print all lines of text
        textarray.forEach(txtline => {
            txtline = txtline.trim();
            ctx.fillText(txtline, textanchor, txtY);
            txtY += charHeight;
        });
    }
    /**
     * Get the height of a canvas text.
     * @param ctx - Canvas context.
     * @param text - Text to be analyzed.
     * @param style - Text style.
     */
    static getTextHeight(ctx, text, style) {
        const previousTextBaseline = ctx.textBaseline;
        const previousFont = ctx.font;
        ctx.textBaseline = 'bottom';
        ctx.font = style;
        const { actualBoundingBoxAscent: height1, actualBoundingBoxDescent: height2 } = ctx.measureText(text);
        // Reset baseline
        ctx.textBaseline = previousTextBaseline;
        ctx.font = previousFont;
        return height1 + height2 + 1.7;
    }
    /**
     * Parse a object.
     * @param json - String object to be parsed.
     */
    static loadObject(json) {
        try {
            return JSON.parse(json);
        }
        catch {
            return null;
        }
    }
    /**
     * @param ctx - Canvas context.
     * @param x - X position.
     * @param y - Y position.
     * @param width - Square width.
     * @param height - Square height.
     * @param radius - Square corner radius.
     */
    static molde(ctx, x, y, width, height, radius) {
        ctx.beginPath();
        ctx.moveTo(x + radius, y);
        ctx.lineTo(x + width - radius, y);
        ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
        ctx.lineTo(x + width, y + height - radius);
        ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
        ctx.lineTo(x + radius, y + height);
        ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
        ctx.lineTo(x, y + radius);
        ctx.quadraticCurveTo(x, y, x + radius, y);
        ctx.closePath();
    }
    /**
     * Check whether given arg is number.
     * @param num - String number to be checked.
     * @returns {boolean}
     */
    static isNumber(num) {
        return !isNaN(parseFloat(num));
    }
    /**
     * Check if the given arg is a valid hexadecimal code.
     * @param value - Code to be tested.
     * @returns {boolean}
     */
    static isValidHex(value) {
        if (!value)
            return false;
        return /^([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/.test(value.replace(/(#)/g, ''));
    }
    /**
     * Parses a string-like value.
     * @param text - String to be parsed.
     */
    static parse(text) {
        if (!text)
            return null;
        else if (text === "undefined")
            return undefined;
        else if (text === "null")
            return null;
        else if (text === "true" || text === "false")
            return text === "true";
        else if (text.match(/\d+n/))
            return BigInt(text.replace("n", ""));
    }
    /**
     * Returns the compilation string from an input code.
     * @param d - Interpreter data.
     * @param code - Code to be resolved.
     * @param returnType - Resolution return type.
     * @returns
     */
    static async resolveCode(d, code, returnType = "code") {
        const { comments, interpreter, reverse } = d;
        const compiled = await interpreter.run(code, {
            comments, data: d, reverse
        });
        return (returnType === "code" ? compiled.code ?? "" : compiled);
    }
    /**
     * Converts a integer to decimal fraction.
     * @param value - Number to be converted.
     * @returns {number}
     */
    static resolvePercent(value) {
        return value / 100;
    }
    /**
     * Returns a string representation of a boolean value.
     * @param value - Value to be converted.
     * @returns {string}
     */
    static stringBool(value) {
        return String(value);
    }
}
exports.Util = Util;
