"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FunctionManager = void 0;
const tslib_1 = require("tslib");
const APIFunction_1 = require("../structures/APIFunction");
const path_1 = require("path");
const fs = tslib_1.__importStar(require("fs"));
class FunctionManager extends Map {
    #injections;
    constructor(iterable) {
        super(iterable);
        this.#injections = {};
    }
    /**
     * Add a function to the manager.
     * @param name - Function name.
     * @param value - Function data.
     */
    add(name, value) {
        return super.set(name.startsWith("$")
            ? name.toLowerCase()
            : "$".concat(name.toLowerCase()), value);
    }
    /**
     * Injects a subfunction to a supported function.
     * @param target - Target function name.
     * @param name - Subfunction name.
     * @param value - Subfunction value.
     */
    inject(target, name, value) {
        target = target.startsWith('$') ? target : "$" + target;
        name = name.startsWith('$') ? name : "$" + name;
        this.#injections[target.toLowerCase()] = {
            [`${name.toLowerCase()}`]: value
        };
    }
    /**
     * Return all function available for a supported function.
     * @param target - Target function name.
     */
    getTargettedInjections(target) {
        target = target.startsWith('$') ? target : "$" + target;
        return this.#injections[target.toLowerCase()];
    }
    /**
     * Load all functions inside a folder.
     * @param dir - Function directory.
     * @param predicate - Predicate to allow loading functions.
     */
    load(dir = __dirname.replace("managers", "functions"), predicate) {
        if (dir.includes(process.cwd()))
            dir = dir.replace(process.cwd(), "");
        const root = process.cwd();
        const files = fs.readdirSync((0, path_1.join)(root, dir));
        for (const file of files) {
            if (file.endsWith("d.ts"))
                continue;
            const isFile = fs.lstatSync((0, path_1.join)(root, dir, file)).isFile() && file.endsWith(".js");
            if (isFile) {
                const File = require((0, path_1.join)(root, dir, file));
                const fn = new File.default;
                if (!fn)
                    continue;
                fn.init();
                if (predicate !== undefined && predicate(fn) === true) {
                    this.add(fn.name, fn);
                    if (fn.aliases?.length) {
                        fn.aliases.forEach((alias) => this.add(alias, fn));
                    }
                }
                else {
                    if (fn.parent instanceof APIFunction_1.APIFunction) {
                        continue;
                    }
                    this.add(fn.name, fn);
                    if (fn.aliases?.length) {
                        fn.aliases.forEach((alias) => this.add(alias, fn));
                    }
                }
            }
            else {
                this.load((0, path_1.join)(dir, file));
                continue;
            }
        }
    }
    /**
     * Returns an array of functions.
     * @returns {APIFunction[]}
     */
    toArray() {
        return Array.from(super.values());
    }
    /**
     * Return all function injections.
     */
    get injections() {
        return this.#injections;
    }
}
exports.FunctionManager = FunctionManager;
