"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.API = void 0;
const tslib_1 = require("tslib");
const FunctionManager_1 = require("../managers/FunctionManager");
const main_1 = require("../main");
const Interpreter_1 = require("../classes/Interpreter");
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
const types_1 = require("util/types");
const package_json_1 = require("../../package.json");
const Addon_1 = require("./Addon");
const F = tslib_1.__importStar(require("fastify"));
const colors_1 = tslib_1.__importDefault(require("colors"));
const path_1 = require("path");
const fs = tslib_1.__importStar(require("fs"));
const Logger_1 = require("../classes/Logger");
class API extends tiny_typed_emitter_1.TypedEmitter {
    options;
    app;
    functions = new FunctionManager_1.FunctionManager();
    interpreter = new Interpreter_1.Interpreter();
    db = new main_1.DatabaseManager;
    constructor(options) {
        super();
        this.options = options;
        this.app = F.fastify(options);
        this.options.reverse = options.reverse ?? false;
        // Add-ons validation.
        if (options.addons?.length) {
            for (const addon of options.addons) {
                this.#validateAddon(addon);
            }
        }
    }
    /**
     * Adds a route to the API.
     */
    route(data) {
        const reverse = data.reverse ?? this.options.reverse ?? false;
        console.log(`${colors_1.default.green(new Date().toLocaleString())} | ${colors_1.default.green("Endpoint loaded")}: ${colors_1.default.bold(`${data.url} => ${data.method.toUpperCase()}`)}`);
        this.app.route({ ...data,
            handler: async (req, res) => {
                try {
                    const runtime = new main_1.Data(reverse, this.functions.entries(), {
                        server: this,
                        request: req,
                        reply: res,
                        stream: null
                    })
                        .setStart(performance.now());
                    const r = await this.interpreter.run(data.code, {
                        comments: data.comments,
                        data: runtime,
                        reverse
                    });
                    this.emit("request", r);
                    if (r.stream)
                        return r.stream;
                }
                catch (error) {
                    console.log(error);
                    throw error;
                }
            }
        });
    }
    /**
     * Run code when events gets emitted.
     */
    event(data) {
        const reverse = data.reverse ?? this.options.reverse ?? false;
        const comments = data.comments ?? false;
        console.log(`${colors_1.default.green(new Date().toLocaleString())} | ${colors_1.default.green("Event loaded")}: ${colors_1.default.bold(data.name)}`);
        this.on(data.name, async (...params) => {
            try {
                const runtime = params[0] instanceof main_1.Data ? params[0] : new main_1.Data(reverse, this.functions.entries(), {
                    server: this,
                    request: null,
                    reply: null,
                    stream: null
                });
                await this.interpreter.run(data.code, {
                    comments, reverse, data: runtime
                });
            }
            catch (error) {
                console.log(error);
                throw error;
            }
        });
    }
    /**
     * Loads routes, events and functions using this method.
     */
    load(dir) {
        const mdir = process.cwd();
        fs.readdirSync((0, path_1.join)(mdir, dir)).forEach((file) => {
            if (fs.lstatSync((0, path_1.join)(mdir, dir, file)).isDirectory()) {
                this.load((0, path_1.join)(dir, file));
            }
            else {
                const all = require((0, path_1.join)(mdir, dir, file));
                if (Array.isArray(all)) {
                    all.forEach(d => {
                        if (d.route && this.#isRoute(d.route))
                            this.route(d.route);
                        else if (d.event && this.#isEvent(d.event))
                            this.event(d.event);
                        else if (d.data && this.#isFunction(d.data))
                            this.functions.add(file.slice(0, -3), d.data);
                    });
                }
                else {
                    if (all.route && this.#isRoute(all.route))
                        this.route(all.route);
                    else if (all.event && this.#isEvent(all.event))
                        this.event(all.event);
                    else if (all.data && this.#isFunction(all.data))
                        this.functions.add(file.slice(0, -3), all.data);
                }
            }
        });
    }
    #isEvent(data) {
        return "name" in data && typeof data.name === "string" && "code" in data && typeof data.code === "string";
    }
    #isRoute(data) {
        return "url" in data && typeof data.url === "string" && "code" in data && typeof data.code === "string";
    }
    #isFunction(data) {
        return "run" in data && (0, types_1.isAsyncFunction)(data.run);
    }
    #validateAddon(addon) {
        // Validating addon inheritance.
        if (!(addon instanceof Addon_1.Addon)) {
            throw new main_1.EATS_Error([
                "Invalid add-on provided!",
                `"${addon.constructor.name}" must be instance of Addon!`
            ].join("\n"));
        }
        // Required version validation.
        if (addon.requiredVersions.length > 0 && !addon.requiredVersions.includes(package_json_1.version)) {
            throw new main_1.EATS_Error([
                `Unsupported easy-api.ts version by the next plugin: "${addon.name}"`,
                `Make sure to use one of the following versions: ${addon.requiredVersions.join(", ")}`
            ].join("\n"));
        }
        // Extra addon validation.
        for (const requiredAddon of addon.requiredAddons) {
            const addons = this.options.addons?.map(x => x.name) ?? [];
            if (!addons.includes(requiredAddon)) {
                throw new main_1.EATS_Error([
                    `"${addon.name}" required the next plugin: "${requiredAddon}"`
                ].join("\n"));
            }
        }
    }
    connect(options) {
        // Crash handler.
        process.on("uncaughtException", (except) => {
            this.emit("error", except);
        });
        // Starting all add-ons.
        for (const addon of (this.options.addons ?? [])) {
            addon.init(this);
            Logger_1.Logger.log(Logger_1.LogType.Info, `Extension loaded${colors_1.default.reset("")}: ${colors_1.default.bold(addon.name)}`);
        }
        this.functions.load();
        const reverse = this.options.reverse ?? false;
        const runtime = new main_1.Data(reverse, this.functions.entries(), {
            server: this,
            request: null,
            reply: null,
            stream: null
        })
            .setStart(performance.now());
        // Starting the server.
        this.app.listen(options).then(() => {
            this.emit("ready", runtime);
            // Beauty log uwu
            Logger_1.Logger.log(Logger_1.LogType.Info, `Listening the server at PORT:${colors_1.default.reset("")} ${colors_1.default.bold(colors_1.default.yellow(options.port?.toString() ?? "Not defined"))}`);
            Logger_1.Logger.log(Logger_1.LogType.Start, require("../../package.json").version);
        });
    }
    /**
     * Whether "rs" syntax is enabled.
     */
    get rs() {
        return this.options.dots ?? false;
    }
}
exports.API = API;
