"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Data = void 0;
const FunctionManager_1 = require("../managers/FunctionManager");
const Interpreter_1 = require("../classes/Interpreter");
class Data {
    server;
    break;
    code;
    comments;
    functions;
    function;
    interpreter;
    parent;
    req;
    res;
    reverse;
    startTime;
    stream;
    #rs;
    #_vars;
    #_internal_vars;
    #locale;
    #timeZone;
    #date;
    constructor(reverse, functions, apiData, locale) {
        this.server = apiData?.server || null;
        this.break = false;
        this.code = null;
        this.comments = false;
        this.functions = new FunctionManager_1.FunctionManager(functions);
        this.function = {
            name: "",
            inside: "",
            fields: []
        };
        this.interpreter = new Interpreter_1.Interpreter();
        this.parent = null;
        this.req = apiData?.request || null;
        this.res = apiData?.reply || null;
        this.reverse = reverse;
        this.startTime = performance.now();
        this.stream = apiData?.stream || null;
        this.#_vars = {};
        this.#_internal_vars = {};
        this.#rs = false;
        this.#locale = locale?.locale ?? "en-US";
        this.#timeZone = locale?.timeZone ?? "America/New_York";
        const dateString = new Date().toLocaleDateString(this.#locale, {
            timeZone: this.#timeZone
        });
        this.#date = new Date(dateString);
    }
    /**
     * Deletes a variable from the cached data.
     * @param name - Variable name.
     * @returns {Data}
     */
    deleteVar(name) {
        delete this.#_vars[name];
        return this;
    }
    /**
     * Deletes an internal variable from the cached data.
     * @param name - Variable name.
     * @returns {Data}
     */
    deleteInternalVar(name) {
        delete this.#_internal_vars[name];
        return this;
    }
    /**
     * Extend this parent data.
     * @param vars - Variables to be added to this child data.
     * @returns {Data}
     */
    extend(vars, internals) {
        const newData = new Data(this.reverse, this.functions.entries(), {
            server: this.server,
            request: this.req,
            reply: this.res,
            stream: this.stream
        })
            .setParent(this)
            .setStart(this.startTime)
            .stopCode(this.break);
        // Cloning cached vars.
        if (vars) {
            for (const [key, value] of Object.entries(vars)) {
                newData.setVar(key, value);
            }
        }
        // Cloning internal vars.
        if (internals) {
            for (const [key, value] of Object.entries(internals)) {
                newData.setInternalVar(key, value);
            }
        }
        return newData;
    }
    /**
     * Check whether a internal variable is cached.
     * @param name - Variable name.
     * @returns {boolean}
     */
    internalVarExists(name) {
        return Object.prototype.hasOwnProperty.call(this.#_internal_vars, name);
    }
    /**
     * Overrides or add internal variables in this data.
     * @param vars - Variables to override.
     * @returns {Data}
     */
    overrideInternals(internals) {
        if (internals === null) {
            this.#_internal_vars = {};
        }
        else {
            for (const [key, value] of Object.entries(internals)) {
                this.setInternalVar(key, value);
            }
        }
        return this;
    }
    /**
     * Overrides or add variables in this data.
     * @param vars - Variables to override.
     * @returns {Data}
     */
    overrideVars(vars) {
        if (vars === null) {
            this.#_vars = {};
        }
        else {
            for (const [key, value] of Object.entries(vars)) {
                this.setVar(key, value);
            }
        }
        return this;
    }
    /**
     * Set a internal variable.
     * @param name - Variable name.
     * @param value - Variable value.
     * @returns {Data}
     */
    setInternalVar(name, value) {
        if (this.parent !== null && this.parent.internalVarExists(name)) {
            this.parent.setInternalVar(name, value);
        }
        this.#_internal_vars[name] = value;
        return this;
    }
    /**
     * Set the date locale.
     * @param tz - Locale to be set.
     * @returns {Data}
     */
    setLocale(locale) {
        this.#locale = locale;
        return this;
    }
    /**
     * Set the date timezone.
     * @param tz - Timezone to be set.
     * @returns {Data}
     */
    setTimeZone(tz) {
        this.#timeZone = tz;
        return this;
    }
    /**
     * Set a variable in the cached data.
     * @param name - Variable name.
     * @param value - Variable value.
     * @returns {Data}
     */
    setVar(name, value) {
        if (this.parent !== null && this.parent.varExists(name)) {
            this.parent.setVar(name, value);
        }
        this.#_vars[name] = value;
        return this;
    }
    /**
     * Retrieves a internal variable.
     * @param name - Variable name.
     * @returns {T}
     */
    getInternalVar(name) {
        return this.#_internal_vars[name] ?? null;
    }
    /**
     * Retrieves a cached variable.
     * @param name - Variable name.
     * @returns {T}
     */
    getVar(name) {
        return this.#_vars[name] ?? null;
    }
    /**
     * Set a parent data to this.
     * @param parent - Parent data.
     * @returns {Data}
     */
    setParent(parent) {
        this.parent = parent;
        return this;
    }
    /**
     * Set the code execution start time.
     * @param time - Start time.
     * @returns {Data}
     */
    setStart(time) {
        this.startTime = time;
        return this;
    }
    /**
     * Whether stop code execution.
     * @param state - Stop state.
     * @returns {Data}
     */
    stopCode(state = true) {
        this.break = state;
        return this;
    }
    /**
     * Check whether a variable is cached.
     * @param name - Variable name.
     * @returns {boolean}
     */
    varExists(name) {
        return Object.prototype.hasOwnProperty.call(this.#_vars, name);
    }
    /**
     * Returns the current date using local time.
     */
    get date() {
        return this.#date;
    }
    /**
     * Whether the core has "rs" syntax enabled.
     */
    get dots() {
        return this.#rs;
    }
    /**
     * Returns the system locale.
     */
    get locale() {
        return this.#locale;
    }
    /**
     * Returns the system timezone.
     */
    get timeZone() {
        return this.#timeZone;
    }
    /**
     * Returns the cached variables.
     */
    get _() {
        return this.#_vars;
    }
    /**
     * Returns the cached internal variables.
     */
    get __() {
        return this.#_internal_vars;
    }
}
exports.Data = Data;
